/*Logomatic V02


 */
/*
  Extensive rewrite of several sections.

  Removed 10 copies of same code for ADC conversion and created a
  general purpose subroutine to do the task.

  Moved all of the interrupt time output functions to a single 
  subroutine that handles things a byte at a time.

  Replaced the three copies of the foreground buffering code with
  a single version. Also generalized the buffer code so that N
  buffers can be used. Limited only by available memory.

  Included a switch to idle mode in the main data loop. The power savings
  isn't as great as with some micro-controllers but it does save some. Dependent
  on interrupt frequency of course.

  Function should be the same with one exception. Mode 1 no longer pads each
  512 byte sector but records the data as a continuous stream.

  The code size is cut by about half. Not really significant given the 
  512K of flash memory. But my FORTH sensibilities are appeased. :-)

  dwschultz Aug 2008

*/

#define FATLIB


/**********************************************************
                  Header files
**********************************************************/
#include "LPC21xx.h"
#include <__armlib.h>
//#include <stdio.h>
#include <stdlib.h>

#ifdef FATLIB
#include "fatlib.h"
#include "HALayer.h"
#else
#include "fat.h"
#endif




#define IAP_LOCATION  0x7FFFFFF1
//#define FIQ_INTERRUPTS

  /* *********************************************************
  Function declarations
  ********************************************************* */
void init_buffers(void);
int out_byte( char val );
void handle_data(void);
void blink_forever( unsigned long count );
void do_adc( int channel );
void __putchar(int ch);




void Initialize(void);
void feed(void);

void setup_uart0(int baud, char want_ints);
void setup_uart1(int baud, char want_ints);

void mode_0(void);
void mode_1(void);
void mode_2(void);
void mode_2_config(void);

void Log_init(void);
void test(void);

char getc(void);

void stat0_on (void);
void stat0_off (void);
void stat1_on (void);
void stat1_off (void);

void put_char(char);
void putc2(char);
char getc2(void);



//void T0isr(void)	__irq;
void IRQ_Routine (void)   __attribute__ ((interrupt("IRQ")));
static void UART0ISR(void) __attribute__ ((interrupt ("IRQ")));
static void UART0ISR_2(void) __attribute__ ((interrupt ("IRQ")));
static void UART1ISR(void) __attribute__ ((interrupt ("IRQ")));
static void MODE2ISR(void) __attribute__ ((interrupt ("IRQ")));

void FIQ_Routine (void)   __attribute__ ((interrupt("FIQ")));
void SWI_Routine (void)   __attribute__ ((interrupt("SWI")));
void UNDEF_Routine (void) __attribute__ ((interrupt("UNDEF")));




/**********************************************************
                  Global Variables
**********************************************************/
char port = 0;
signed char handle;
char title[10];

//Default settings=================================================================================
static char mode = 0;    //uart mode, 0 = uart auto, 1 = uart trigger, 2 = adc
static char asc = 0;    //log in ascii mode
static int baud = 9600;   //baud = 9600
static int freq = 100;  //100Hz default for ADC operations
static char trig = '$';  //standard start of nmea sentences
static short frame = 100; //100 characters after trigger character IF it's in uart auto mode
static char ad1_5 = 0;   //off
static char ad1_4 = 0;   //off
static char ad1_3 = 0;   //off
static char ad0_3 = 0;   //off
static char ad0_2 = 0;   //off
static char ad0_1 = 0;   //off
static char ad1_2 = 0;   //off
static char ad0_4 = 0;   //off
static char ad1_7 = 0;   //off
static char ad1_6 = 0;   //off

/*
  Short routine that does nothing but blink the status lights forever.
 */

void blink_forever( unsigned long count )
{
  unsigned long j;

  while(1)
    {
      stat0_on();
      for (j = 0; j < count; j++);
      stat0_off();
      stat1_on();
      for (j = 0; j < count; j++);
      stat1_off();
    }
}

/*
  Data buffers.

  These buffers contain storage for 512 bytes of data and a counter.
  The counter serves double duty as a buffer full flag. When it equals
  512 the foreground program will write it to main storage and reset the
  counter to zero.
*/
#define BUF_SIZE 512
#define NUM_BUFFERS 32

struct {
  unsigned char data[BUF_SIZE];
  int  count;
} buffer [NUM_BUFFERS];

/*
  Interrupt time routine to output a single byte. Checks for a full
  buffer and switches to the next in line.

  If a buffer over run occurs (no space for data) an error condition is returned 
  and the data is not saved.
 */
int out_byte( char val )
{
  static int buf_index = 0;

  /* The current buffer should have one open space. In a buffer overrun
     condition it will not. Check for that and return an error.
  */
  if(buffer[buf_index].count == BUF_SIZE)
    return -1;

  /* Store the data.
   */
  buffer[buf_index].data[buffer[buf_index].count++] = val;

  /* Now check to see if the buffer is full. */

  if(buffer[buf_index].count == BUF_SIZE)
    {
      /* Step to the next buffer. Checking to see
	 if it is the last one.
      */
      buf_index++;
      if(buf_index >= NUM_BUFFERS)
	buf_index = 0;
    }
  return 0;
}

/*
  Mark all data buffers as empty.
 */
void init_buffers(void)
{
  int i;

  for(i = 0; i < NUM_BUFFERS; i++)
    buffer[i].count = 0;
}

/*
  This is the foreground routine to handle data.

  Check the current buffer to see if it is full. If it is, issue
  a write command through the FAT file system code. Then mark it as empty.

  The original code called fat_flush() after each block. This caused the directory
  entry to be written as well. Two writes for every block. Removing that improves the average
  speed but there will still be the occasional stall when the FAT codes decides to
  update things on its own. Not to mention the two writes required to get a new
  cluster.

  If the stop button is pressed, the file is closed and then everything
  stops.
 */
void handle_data(void)
{

  int buf_index;

  buf_index = 0;
  while(1)
    {
      if(buffer[buf_index].count == BUF_SIZE)
	{
	  /*
	    Blink the status lights in alternation. If the number of buffers
	    is even, this will repeat the stat0 blink each time around.
	   */
	  (buf_index & 1) ? stat0_on() : stat1_on();
	  /*
	    The current action if the write fails is to stop immediately. This is
	    probably not the best thing to do.
	   */
#ifdef FATLIB
	  if (fat_write(handle,buffer[buf_index].data, BUF_SIZE) < 0) //stop if we can't write
	      blink_forever(500000);
#else
	  if (fat_write(buffer[buf_index].data) ) //stop if we can't write
	      blink_forever(500000);
#endif
	  (buf_index & 1) ? stat0_off() : stat1_off();
	  buffer[buf_index].count = 0;
	  if(++buf_index >= NUM_BUFFERS)
	    buf_index = 0;
        }

        
      //if the button has been pushed, log the rest of the file and quit
      if ((IOPIN0 & 0x00000008) == 0)    
        {
	  VICIntEnClr = 0xFFFFFFFF;  //clear all interrupts

	  /* There are several cases to handle.
	     1) buffers empty, nothing to do.
	     2) Current buffer partially full.
	     3) Several buffers full/partially full.

	     One loop handles them all.
	  */
	  while (buffer[buf_index].count != 0)
	    {
	      if( buffer[buf_index].count != BUF_SIZE )
		{
		  /* Pad buffer with nulls. */
		  while(buffer[buf_index].count != BUF_SIZE)
		    out_byte(0);
		}
#ifdef FATLIB
	      fat_write(handle,buffer[buf_index].data, BUF_SIZE);
#else
	      fat_write(buffer[buf_index].data);
#endif
	      buffer[buf_index].count = 0;
	      if(++buf_index >= NUM_BUFFERS)
		buf_index = 0;
	    }
#ifdef FATLIB
	  fat_close(handle);
          fat_flush();
#else
	  fat_close(buffer[0].data);
#endif
    
	  blink_forever(500000);
        }
      /* This would be an excellent spot to put the processor in idle mode.
	 Since there is zero chance of a buffer spontaneously filling itself without
	 an interrupt from either the timer or UART, we can sleep. The interrupt will
	 start the ball rolling again.

	 This will decrease the average power used. According to the data sheet normal current
	 is about 40mA while idle current is about 10mA.
      */
      PCON |= 1;      /* Sleep until an interrupt occurs */
    }


}


void __putchar(int ch)
{
  if (ch == '\n')
    put_char('\r');
  put_char(ch);
}

/**********************************************************
                       MAIN
**********************************************************/

int	main (void)
{
  __ARMLIB_enableFIQ();

  //iap_entry=(IAP)IAP_LOCATION;

  int i, j;
    
  Initialize();
  init_buffers();

#ifdef FATLIB    
  if(fat_initialize() != 1)   // Initialise the FAT library.
    blink_forever(500000);
#else
  fat_init(buffer[0].data);
#endif
  setup_uart0(9600, 0);

  for (i = 0; i < 5; i++)
    {
      stat0_on();
      for (j = 0; j < 120000; j++);
      stat0_off();
      stat1_on();
      for (j = 0; j < 120000; j++);
      stat1_off();
    }

  /* Read the configuration file. */

  Log_init();

  /* Open the output file. */

  for (i = 0; i < 255; i++)
    {
#ifdef FATLIB
      sprintf(title,"LOG%d.txt",i);
      handle = fat_openWrite(title);
      if (handle >= 0) break;
#else
      sprintf(title, " LOG%04dTXT", i);
      j = fat_create(title, buffer[0].data);
      if(j == -1)
	blink_forever(240000);
      if(j == -2)
	continue;
#endif
    }
#ifdef FATLIB
  if (handle < 0) //If we can't open a file, lock up right here...===============================
    blink_forever(240000);
#endif
  /* Setup the selected mode */

  if (mode == 0)
    setup_uart0(baud, 1);
  else if (mode == 1)
    setup_uart0(baud, 2);
  else if (mode == 2)
    mode_2_config();

  /* Process and store the data. */

  handle_data();
}

/**********************************************************
                      Initialize
**********************************************************/

#define PLOCK 0x400

void Initialize(void)
{

  // Setting Multiplier and Divider values
  /* 
     Fosc = 14.7456MHZ
     M = 4
     P = 2
     CCLK = 4 * 14.7456MHz = 58.9824MHz
     Fcco = 2 * 2 * CCLK = 235.9296MHZ
  */
  PLLCFG=0x23;
  feed();
  
  // Enabling the PLL */	
  PLLCON=0x1;
  feed();
  
  // Wait for the PLL to lock to set frequency
  while(!(PLLSTAT & PLOCK)) ;
  
  // Connect the PLL as the clock source
  PLLCON=0x3;
  feed();
  
  // Enabling MAM and setting number of clocks used for Flash memory fetch (4 cclks in this case)
  MAMCR=0x2;
  MAMTIM=0x4;
  
  // Setting peripheral Clock (pclk) to System Clock (cclk)
  VPBDIV=0x01;
	
  PINSEL0 = 0xCF351505;     // enable uart0, enable uart1, enable SPI0 but with SSEL as IO, AD1.2,1.3,1.4 1.5 active
  PINSEL1 = 0x15441801;   //AD0.1,0.2,0.3,0.4,1.6,1.7 active
  // Don't have to set the direction of the UART pins after they're configured with PINSEL0
                                        
  IODIR0 |= 0x00000884;   //


  IOSET0 = 0x00000080;	//	

  //set up the SPI port
  S0SPCCR = 0x08;			// set up SPI clk to be pclk/8, this is the fastest we can go
  S0SPCR = 0x30;			//master, msb first, first clk edge, active high, no ints
	
}

/*
  Modifying the PLL settings requires that a special sequence be written to
  the PLLFEED register. This cannot be interrupted by interrupts.
*/
void feed(void)
{
  PLLFEED=0xAA;
  PLLFEED=0x55;
}






/*  Stubs for various interrupts (may be replaced later)  */
/*  ----------------------------------------------------  */

static void
UART0ISR(void)
{
  char temp;


  out_byte(U0RBR);
      
  temp = U0IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;

}

/*
  Handled the UART interrupt when in triggered mode. Scans for the 
  trigger character and records the specified number of characters after it.
  It also records the trigger character and appends a carriage return - line feed
  pair at the end.
*/
static void UART0ISR_2(void)
{
  char temp;


  static int count = 0;

  temp = U0RBR;

  if(count == 0)
    if(temp == trig)
      count = frame;

  if( count )
    {
      out_byte(temp);
      count--;
      if(count == 0)
	{
	  out_byte('\n');
	  out_byte('\r');
	}
    }
      
  temp = U0IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;
}

/*
  It seemed foolish to repeat the same code over and over again for
  each ADC conversion so I have created a subroutine to handle it.

  Call with the channel number.

  Channel numbers 0 to 7 use ADC0 and 8 to 15 use ADC1

*/
void do_adc( int channel )
{
  long temp;
  char temp_buf[10];
  int i;

  /* Select channel, clock rate, etc. */

  if(channel >= 0 && channel < 8)   // Use ADC 0
    {
      AD0CR = 0x0020FF00 | 1 << channel;

      AD0CR |= 0x01000000;//start conversion
            
      do {
	temp = AD0DR;
      }
      while ((temp & 0x80000000) == 0);
    }
  else if( channel > 7 && channel < 16) // Use ADC 1
    {
      AD1CR = 0x0020FF00 | 1 << (channel - 8);

      AD1CR |= 0x01000000;//start conversion
            
      do {
	temp = AD1DR;
      }
      while ((temp & 0x80000000) == 0);
    }

  /* The result is in bits 15:6 so shift right and mask */
  temp >>= 6;
  temp &= 0x3ff;

  if (asc)
    {
      itoa(temp, temp_buf, 10);

      for(i = 0; i < 4; i++)
	if(temp_buf[i] >= '0')
	  out_byte( temp_buf[i]);
      out_byte('\t');    /* Put a tab character between values */
    }       
  else
    {
      out_byte(temp >> 8);
      out_byte(temp);
    }
}

static void
MODE2ISR(void)
{



    
  T0IR = 1;   //reset TMR0 interrupt
    
  /* Perform the ADC conversions as required
     This probably should be redone to make it table driven
     instead of using this ugly series of if() statements.
   */
    
  if (ad1_5)
      do_adc(8+5);

  if (ad1_4 )
      do_adc(8+4);

  if (ad1_3 )
      do_adc(8+3);

  if (ad0_3)
      do_adc(3);

  if (ad0_2)
      do_adc(2);

  if (ad0_1)
      do_adc(1);

  if (ad1_2)
      do_adc(8+2);

  if (ad0_4)
      do_adc(4);

  if (ad1_7)
      do_adc(8+7);

  if (ad1_6)
    do_adc(8+6);

  /*
    Output some end of line delimiters. A carriage return - line feed
    pair in ASCII mode and "$$" in binary.
  */
  if(asc)
    {
      out_byte('\r');
      out_byte('\n');
    }
  else
    {
      out_byte('$');
      out_byte('$');
    }
	
  VICVectAddr = 0;
	
}
    
static void
UART1ISR(void)
{
  char temp;
  int j;

  stat0_on();
  for (j = 0; j < 750000; j++);
  stat0_off();
      
  temp = U1RBR;

      
  temp = U1IIR;	//have to read this to clear the int

  /* Update VIC priorities */
  VICVectAddr = 0;

}



void FIQ_Routine (void)
{
  char a;
  int j;
        
  stat0_on();
  for (j = 0; j < 5000000; j++);
  stat0_off();
  a = U0RBR;

  a = U0IIR;	//have to read this to clear the int
  //ILR = 3;
        
}


void SWI_Routine (void)  {
  while (1) ;	
}


void UNDEF_Routine (void) {
	
  stat0_on();
  //while (1) ;	
}

//Sends nate to the Transmit Register
void put_char(char c)
{
  if (port == 0)
    {
      while((U0LSR & 0x20) == 0);
      U0THR = c;
    }
  else if (port == 1)
    {
      while((U1LSR & 0x20) == 0);
      U1THR = c;
    }
}


//specific to UART0
char getc(void)
{
  while((U0LSR & 0x01) == 0);
  return (U0RBR);
}
   
char getc2(void)
{
  while((U1LSR & 0x01) == 0);
  return (U1RBR);
}


void putc2(char c)
{
  while((U1LSR & 0x20) == 0);
  U1THR = c;
}

void stat0_on (void)
{
  IOCLR0 = 0x00000004;

}

void stat0_off (void)
{
  IOSET0 = 0x00000004;
	
}

void stat1_on (void)
{
  IOCLR0 = 0x00000800;

}

void stat1_off (void)
{
  IOSET0 = 0x00000800;

}



void setup_uart0(int baud, char want_ints)
{
  //set up uart0 
  U0LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
  if (baud == 1200)
    {
      U0DLM   = 0x0C;		  	//
      U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
	
  if (baud == 2400)
    {
      U0DLM   = 0x06;		  	//
      U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
		
  if (baud == 4800)
    {
      U0DLM   = 0x03;		  	//
      U0DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 9600)
    {
      U0DLM   = 0x01;		  	//
      U0DLL 	= 0x80;        	// 9600 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 19200)
    {
      U0DLM   = 0x00;		  	//
      U0DLL 	= 0xC0;        	// 19200 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 38400)
    {
      U0DLM   = 0x00;		  	//
      U0DLL 	= 0x60;        	// 38400 Baud Rate @ 58982400 VPB Clock
    }

  else if (baud == 57600)
    {
      U0DLM   = 0x00;		  	//
      U0DLL 	= 0x40;        	// 57600 Baud Rate @ 58982400 VPB Clock
    }

  else if (baud == 115200)
    {
      U0DLM   = 0x00;		  	//
      U0DLL 	= 0x20;        	// 115200 Baud Rate @ 58982400 VPB Clock
    }
	                    
  U0FCR	= 0x01;			// Not sure this is necessary, but the manual sez it is
  U0LCR 	= 0x03;        	// DLAB = 0

  if (want_ints == 1)
    {
      //__ARMLIB_enableFIQ();
      __ARMLIB_enableIRQ();

      //stat1_on();
      /* UART0 interrupt is an IRQ interrupt */
      //VICIntSelect |= 0x00000040;
      VICIntSelect &= ~0x00000040;
      /* Enable UART interrupt */
      VICIntEnable |= 0x00000040;
      /* Use slot 1 for UART0 interrupt */
      VICVectCntl1 = 0x26;
      /* Set the address of ISR for slot 1 */
      VICVectAddr1 = (unsigned int)UART0ISR;

            

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U0IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }

  else if (want_ints == 2)
    {
      //__ARMLIB_enableFIQ();
      __ARMLIB_enableIRQ();

      //stat1_on();
      /* UART0 interrupt is an IRQ interrupt */
      //VICIntSelect |= 0x00000040;
      VICIntSelect &= ~0x00000040;
      /* Enable UART interrupt */
      VICIntEnable |= 0x00000040;
      /* Use slot 1 for UART0 interrupt */
      VICVectCntl2 = 0x26;
      /* Set the address of ISR for slot 1 */
      VICVectAddr2 = (unsigned int)UART0ISR_2;

            

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U0IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }
	
  else if (want_ints == 0)
    {
      VICIntEnClr = 0x00000040;	//disables UART0 interupt
      U0IER = 0x00;					//disable RX data available interupt, DLAB must be 0 to write
    }
	
      
}


void setup_uart1(int baud, char want_ints)
{
  char q;

  //set up uart1
  U1LCR 	= 0x83;			// 8 bits, no Parity, 1 Stop bit, DLAB = 1
	
  if (baud == 1200)
    {
      U1DLM   = 0x0C;		  	//
      U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
	
  if (baud == 2400)
    {
      U1DLM   = 0x06;		  	//
      U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
	
  if (baud == 4800)
    {
      U1DLM   = 0x03;		  	//
      U1DLL 	= 0x00;        	// 4800 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 9600)
    {
      U1DLM   = 0x01;		  	//
      U1DLL 	= 0x80;        	// 9600 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 19200)
    {
      U1DLM   = 0x00;		  	//
      U1DLL 	= 0xC0;        	// 19200 Baud Rate @ 58982400 VPB Clock
    }
	
  else if (baud == 38400)
    {
      U1DLM   = 0x00;		  	//
      U1DLL 	= 0x60;        	// 38400 Baud Rate @ 58982400 VPB Clock
    }
  //U1IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write               
  U1FCR	= 0x03;			// Not sure this is necessary, but the manual sez it is
  	
  U1LCR 	= 0x03;        	// DLAB = 0
	
  q = U1RBR;


  if (want_ints == 1)
    {
      //stat1_on();
      /* UART1 interrupt is an IRQ interrupt */
      VICIntSelect &= ~0x80;
      /* Enable UART interrupt */
      VICIntEnable = 0x80;
      /* Use slot 0 for UART1 interrupt */
      VICVectCntl0 = 0x27;
      /* Set the address of ISR for slot 0 */
      VICVectAddr0 = (unsigned int)UART1ISR;

      __ARMLIB_enableIRQ();

      //VICIntSelect |= 0x00000080;		//bit 7 is for UART1, sets to FIQ (not IRQ)
      //VICIntEnable |= 0x00000080;	//enables UART1 interupt
      U1IER = 0x01;					//enable RX data available interupt, DLAB must be 0 to write
		
    }
	
  else if (want_ints == 0)
    {
      VICIntEnClr = 0x00000080;	//disables UART1 interupt
      U1IER = 0x00;					//disable RX data available interupt, DLAB must be 0 to write
    }
	

}



void Log_init(void)
{
  int j, x, mark = 0, index = 0;
  char temp, temp2 = 0;
  signed int stringSize;
  signed char handle;
  unsigned char stringBuf[BUF_SIZE+1], safety;

  handle = fat_openRead("LOGcon.txt");
  if (handle >= 0)
    {
      stringSize = fat_read(handle, stringBuf, BUF_SIZE);
      stringBuf[stringSize] = '\0';
      //printf(stringBuf);
      fat_close(handle);
    }

  else
    {
#ifdef FATLIB
      handle = fat_openWrite("LOGcon.txt");
      if (handle < 0)
	blink_forever(500000);    //in case there's no card or it's full...

      strcpy(stringBuf, "MODE = 0\r\nASCII = N\r\nBaud = 4\r\nFrequency = 100\r\nTrigger Character = $\r\nText Frame = 100\r\nAD1.5 = N\r\nAD1.4 = N\r\nAD1.3 = N\r\nAD0.3 = N\r\nAD0.2 = N\r\nAD0.1 = N\r\nAD1.2 = N\r\nAD0.4 = N\r\nAD1.7 = N\r\nAD1.6 = N\r\nSafty On = Y\r\n");
      stringSize = strlen(stringBuf);
      fat_write(handle,stringBuf, stringSize);

      fat_close(handle);


#else
      if(fat_create("  LOGcontxt", buffer[0].data) != 0)
	blink_forever(500000);

      strcpy(buffer[0].data, "MODE = 0\r\nASCII = N\r\nBaud = 4\r\nFrequency = 100\r\nTrigger Character = $\r\nText Frame = 100\r\nAD1.5 = N\r\nAD1.4 = N\r\nAD1.3 = N\r\nAD0.3 = N\r\nAD0.2 = N\r\nAD0.1 = N\r\nAD1.2 = N\r\nAD0.4 = N\r\nAD1.7 = N\r\nAD1.6 = N\r\nSafty On = Y\r\n");
      fat_write(buffer[0].data);
      fat_close(buffer[0].data);
#endif
    }    
    
  for (x = 0; x < stringSize; x++)
    {
      temp = stringBuf[x];
      if (temp == 10)
        {
	  mark = x;
	  index++;

	  if (index == 1)
            {
	      mode = stringBuf[mark - 2] - 48; //0 = auto uart, 1 = trigger uart, 2 = adc
            }

	  else if (index == 2)
            {
	      if(stringBuf[mark - 2] == 'Y')
		asc++;    //default is 'N'
            }
        
	  else if (index == 3)
            {
	      if (stringBuf[mark - 2] == '1') baud = 1200;
	      else if (stringBuf[mark - 2] == '2') baud = 2400;
	      else if (stringBuf[mark - 2] == '3') baud = 4800;
	      else if (stringBuf[mark - 2] == '4') baud = 9600;
	      else if (stringBuf[mark - 2] == '5') baud = 19200;
	      else if (stringBuf[mark - 2] == '6') baud = 38400;
	      else if (stringBuf[mark - 2] == '7') baud = 57600;
	      else if (stringBuf[mark - 2] == '8') baud = 115200;
                
            }
            
	  else if (index == 4)
            {
	      freq = (stringBuf[mark - 2] - 48) + (stringBuf[mark - 3] - 48) * 10;
	      if ((stringBuf[mark - 4] >= 48) && (stringBuf[mark - 4] < 58))
                {
		  freq += (stringBuf[mark - 4] - 48) * 100;
		  if ((stringBuf[mark - 5] >= 48) && (stringBuf[mark - 5] < 58)) freq += (stringBuf[mark - 5] - 48) * 1000;
                }
            }
    
	  else if (index == 5)
            {
	      trig = stringBuf[mark - 2]; //default is $
                
            }
    
	  else if (index == 6)
            {
	      frame = (stringBuf[mark - 2] - 48) + (stringBuf[mark - 3] - 48) * 10 + (stringBuf[mark - 4] - 48) * 100;  
	      if (frame > 510) frame = 510;//Up to 510 characters
            }

	  else if (index == 7)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_5++;
		  temp2++;
		}
	    }

	  else if (index == 8)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_4++;
		  temp2++;
		}
	    }
	  else if (index == 9)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_3++;
		  temp2++;
		}
	    }
	  else if (index == 10)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad0_3++;
		  temp2++;
		}
	    }
	  else if (index == 11)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad0_2++;
		  temp2++;
		}
	    }
	  else if (index == 12)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad0_1++;
		  temp2++;
		}
	    }
	  else if (index == 13)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_2++;
		  temp2++;
		}
	    }
	  else if (index == 14)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad0_4++;
		  temp2++;
		}
	    }
	  else if (index == 15)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_7++;
		  temp2++;
		}
	    }
	  else if (index == 16)
            {
	      if(stringBuf[mark - 2] == 'Y') //default is 'N'
		{
		  ad1_6++;
		  temp2++;
		}
	    }
	  else if (index == 17)
	    safety = stringBuf[mark - 2]; //default is 'Y'
        }

    }

  if (safety == 'Y')    //make sure the frequency is within limits for ADC mode if safety is on
    {
      if(temp2 == 0 )
	freq = 100;
      else if(freq > 1500/temp2)
	freq = 1500/temp2;
    }

  if (safety == 'T') test();

  //printf("%d\r\n",freq);

}




void mode_2_config(void)
{
  __ARMLIB_enableIRQ();

  //stat1_on();
  /* Timer0 interrupt is an IRQ interrupt */
  VICIntSelect &= ~0x00000010;
  /* Enable Timer0 interrupt */
  VICIntEnable |= 0x00000010;
  /* Use slot 2 for UART0 interrupt */
  VICVectCntl2 = 0x24;
  /* Set the address of ISR for slot 1 */
  VICVectAddr2 = (unsigned int)MODE2ISR;

  T0TCR = 0x00000002;			//Reset counter and prescaler
  T0MCR = 0x00000003;			//On match reset the counter and generate an interrupt

  T0MR0 = 58982400 / freq;

  T0PR = 0x00000000;
	
  T0TCR = 0x00000001;			//enable timer

}

void test(void)
{
#ifdef TEST
  int temp, temp2;
  int j;
  char a;

  printf("Logomatic Test Code:\r\n");
  printf("Hit any key to test ADC lines, hit the stop button to terminate the test.\r\n\n");

  a = getc();
    
    
  while ((IOPIN0 & 0x00000008) == 0x00000008)
    {
      //Get AD1.5=======================================================================
      AD1CR = 0x0020FF20;	//AD1.5
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
        
      printf("%d",temp2);
      put_char(9);
                
        
      //Get AD1.4=======================================================================
      AD1CR = 0x0020FF10;	//AD1.4
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
        
      printf("%d",temp2);
      put_char(9);
            
      //Get AD1.3=======================================================================
      AD1CR = 0x0020FF08;	//AD1.3
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
                
      printf("%d",temp2);
      put_char(9);
            
      //Get AD0.3=======================================================================
      AD0CR = 0x0020FF08;	//AD0.3
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD0.2=======================================================================
      AD0CR = 0x0020FF04;	//AD0.2
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);  
      put_char(9);
              
      //Get AD0.1=======================================================================
      AD0CR = 0x0020FF02;	//AD0.1
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
    
      printf("%d",temp2);
      put_char(9);
                
            
      //Get AD1.2========================================================================
      AD1CR = 0x0020FF04;	//AD0.0, P0.27
      AD1CR |= 0x01000000;	//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD0.4=======================================================================
      AD0CR = 0x0020FF10;	//AD0.4
      AD0CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD0DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD0CR = 0x00000000;
                
      printf("%d",temp2);        
      put_char(9);
            
      //Get AD1.7=======================================================================
      AD1CR = 0x0020FF80;	//AD1.7
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
    
      printf("%d",temp2);
      put_char(9);
                
        
            
      //Get AD1.6=======================================================================
      AD1CR = 0x0020FF40;	//AD1.6
      AD1CR |= 0x01000000;//start conversion
        
      while ((temp & 0x80000000) == 0)
        {
	  temp = AD1DR;
        }
        
      temp &= 0x0000FFC0;
      temp2 = temp / 0x00000040;
        
      AD1CR = 0x00000000;
    
      printf("%d\r\n",temp2);
       
      for (j = 0; j < 800000; j++);
      put_char(10);
      put_char(13);
    }


  printf("\r\nTest Complete.\r\n");
  #endif
  while(1);


}


